const T = ('type-parser');

// Validation: checking data types
//=============================================================================

T.Number.isTypeOf(5); // true

T.Number.isTypeOf('abc'); // false

T.Number.isTypeOf([]); // false

T.Number.isTypeOf({}); // false

T.Number.isTypeOf(function () {}); // false

T.Array.isTypeOf([]); // true

T.Object.isTypeOf([]); // false

T.Timestamp.isTypeOf(123); // true

T.ValidDate.isTypeOf(new Date(123)); // true

T.ValidDate.isTypeOf(new Date('abc')); // false

T.Date.isTypeOf(new Date('abc')); // true


// Conversion: transforming a data type in another
//=============================================================================

T.String.from(5); // typeof === 'string'

T.Date.from(1000000000000); // Date object, 2001-09-09T01:46:40.000Z

T.Array.from('[1, 2, 3]'); // [ 1, 2, 3 ] (array)

T.Function.from('[1, 2, 3]'); // typeof === 'function'


// Registering new types
//=============================================================================

/**
 * @name T
 * @property EXAMPLE_TYPE_MyType1
 */
T.register({
    title: 'EXAMPLE_TYPE_MyType1',
    from: function (v) {
        return 'MyType1_' + v;
    },
    isTypeOf: function (v) { {
        return /^MyType1_/.test(v);
    }},
});

// Converting a string to MyType1
T.EXAMPLE_TYPE_MyType1.from('myString'); // MyType1_myString

// Checking if the previous result is of that type
T.EXAMPLE_TYPE_MyType1.isTypeOf('MyType1_myString'); // true


// Registering new types with options
//=============================================================================

/**
 * @name T
 * @property EXAMPLE_TYPE_MyType2
 */
T.register({
    title: 'EXAMPLE_TYPE_MyType2',
    from: function (v, options) {
        return 'MyType2_' + v + '_' + options.suffix;
    },
    isTypeOf: function (v, options) { {
        return (new RegExp('^MyType2_[^_]+_' + options.suffix + '$')).test(v);
    }},
});

// Converting a string to MyType1
T.EXAMPLE_TYPE_MyType2.from('myString', { suffix: 'ABC' }); // MyType2_myString_ABC

// Checking if the previous result is of that type
T.EXAMPLE_TYPE_MyType2.isTypeOf('MyType2_myString_ABC', { suffix: 'ABC' }); // true


// Registering new types with options and/or callbacks
//=============================================================================

/**
 * @name T
 * @property EXAMPLE_TYPE_MyType3
 */
T.register({
    title: 'EXAMPLE_TYPE_MyType3',
    requireCallback: true,
    from: function (v, options, cb) {
        setTimeout(() => {
            cb('MyType4_' + v);
        }, 100);
    },
    isTypeOf: function (v, options, cb) { {
        setTimeout(() => {
            cb(/^MyType4_/.test(v));
        }, 100);
    }},
});

// Converting a string to MyType1
T.EXAMPLE_TYPE_MyType3.from('myString', (result) => {
    console.log(result); // MyType3_myString
});

// Checking if the previous result is of that type
T.EXAMPLE_TYPE_MyType3.isTypeOf('MyType4_myString', (result) => {
    console.log(result); // true
});