const isArray = require('../validation/isArray');

/**
 * Accepts anything and tries to parse into a JSON array
 *
 * If invalid input:
 * - Strict mode: returns an empty array
 * - Lenient mode: returns the input wrapped in an array
 * - allowUndefined mode: returns undefined
 *
 * @param {*} v
 * @param {TypeParser_Options} (options)
 * @returns {[]|undefined}
 */
module.exports = function (v, options = {}) {

    const mode = ((options.mode || 'strict') + '').toLowerCase();

    // Warning message for unknown modes
    if (!['strict', 'lenient', 'allowundefined'].includes(mode)) {
        throw {
            type: 'warning',
            message: `WARNING: Unknown mode "${mode}" used for toArray`,
            method: 'toArray',
        };
    }

    let object = v;
    if (typeof v === 'string') {
        try {
            object = JSON.parse(v);
        }
        catch (e) {}
    }

    if (isArray(object)) {
        return object;
    }
    else {
        switch(mode) {
            case 'lenient':
                if (options.forceParseJson) {
                    return [object];
                }
                else {
                    return [v];
                }
                break;

            case 'allowundefined':
                return undefined;
                break;

            case 'strict':
            default:
                return [];
                break;
        }
    }
};