const toNumber = require('./toNumber');
const isEmpty = require('../validation/isEmpty');
const isNumber = require('../validation/isNumber');
const isValidDate = require('../validation/isValidDate');

/**
 * Takes any parameter, converts into a timestamp number (number of milliseconds)
 * and converts it to a JavaScript Date object.
 *
 * If invalid input:
 * - Strict mode: returns a Date object, but invalid
 * - allowUndefined mode: returns undefined
 *
 * @param x Input to be converted
 * @param {TypeParser_Options} (options)
 * @return {Object}
 */
module.exports = function (x, options = {}) {
    const mode = ((options.mode || 'strict') + '').toLowerCase();

    // Warning message for unknown modes
    if (!['strict', 'allowundefined'].includes(mode)) {
        throw {
            type: 'warning',
            message: `WARNING: Unknown mode "${mode}" used for toDate`,
            method: 'toDate',
        };
    }

    if (isValidDate(x)) {
        return x;
    }

    if (isEmpty(x)) {
        return mode === 'strict' ? new Date("Invalid date") : undefined;
    }
    else {

        let v = toNumber(x, { allowUndefined: true });
        if (!isNumber(v)) { v = x; }

        try {
            v = JSON.parse(v);
        }
        catch (e) {}

        const d = new Date(v);
        if (isValidDate(d)) {
            return d;
        }
        else {
            return mode === 'strict' ? new Date("Invalid date") : undefined;
        }
    }
};