const isFunction = require('../validation/isFunction');

/**
 * Converts any value to a function
 *
 * - Strict mode: if not possible to convert to a function, returns a function that does
 *   nothing
 * - allowUndefined mode: if not possible to convert to a function, returns undefined
 *
 * @param {*} v
 * @param {TypeParser_Options} options
 * @returns {String}
 */
module.exports = function (v, options = {}) {

    const mode = ((options.mode || 'strict') + '').toLowerCase();

    // Warning message for unknown modes
    if (!['strict', 'allowundefined'].includes(mode)) {
        throw {
            type: 'warning',
            message: `WARNING: Unknown mode "${mode}" used for toFunction`,
            method: 'toFunction',
        };
    }

    if (isFunction(v)) {
        return v;
    }
    else {
        return mode === 'strict' ? function () {} : undefined;
    }
};