const isObject = require('../validation/isObject');

/**
 * Accepts anything and tries to parse into a JSON object.
 *
 * If invalid input:
 * - Strict mode: if not possible to convert into an object, returns an empty object
 * - allowUndefined mode: if not possible to convert into an object, returns undefined
 *
 * @param {*} v
 * @param {TypeParser_Options} (options)
 * @returns {{}}
 */
module.exports = function (v, options = {}) {

    const mode = ((options.mode || 'strict') + '').toLowerCase();

    // Warning message for unknown modes
    if (!['strict', 'allowundefined'].includes(mode)) {
        throw {
            type: 'warning',
            message: `WARNING: Unknown mode "${mode}" used for toObject`,
            method: 'toObject',
        };
    }

    let object = v;
    if (typeof v === 'string') {
        try {
            object = JSON.parse(v);
        }
        catch (e) {}
    }

    if (isObject(object)) {
        return object;
    }
    else {
        return mode === 'strict' ? {} : undefined;
    }
};