const isObject = require('../validation/isObject');
const isArray = require('../validation/isArray');
const isDate = require('../validation/isDate');

/**
 * Converts any value to a string
 *
 * - Strict mode: if not possible to convert to a string, returns an empty string
 * - Lenient mode: will use the default string parser (var + '')
 * - emptyasundefined mode: similar to strict mode, but will return undefined instead
 *   of empty strings
 *
 * @param {*} v
 * @param {TypeParser_Options} options
 * @returns {String}
 */
module.exports = function (v, options = {}) {

    const mode = ((options.mode || 'strict') + '').toLowerCase();

    // Warning message for unknown modes
    if (!['strict', 'lenient', 'emptyasundefined'].includes(mode)) {
        throw {
            type: 'warning',
            message: `WARNING: Unknown mode "${mode}" used for toString`,
            method: 'toString',
        };
    }

    // Lenient mode = default conversion
    if (mode === 'lenient') {
        return (v && v.toString && v.toString()) || '' + v;
    }
    else {
        if (
            // Null or undefined
            (v === null  || v === undefined) ||

            // An object or array, but NOT a date
            ((isObject(v) || isArray(v)) && !isDate(v)) ||

            // NaN
            (typeof v === 'number' && isNaN(v)) ||

            // Function
            (typeof v === 'function')
        ) {
            return mode === 'strict' ? '' : undefined;
        }

        // If it is a date, we return the toDateString
        if (isDate(v)) {
            return v.toDateString();
        }

        return ('' + v).length > 0
             ? ('' + v)
             : mode === 'strict' ? '' : undefined;
    }
};