const toNumber = require('./toNumber');
const isEmpty = require('../validation/isEmpty');
const isNumber = require('../validation/isNumber');
const isValidDate = require('../validation/isValidDate');

/**
 * Takes any parameter and converts into a timestamp number (number of milliseconds)
 *
 * If invalid input:
 * - Strict mode: returns NaN
 * - NaNTo0 mode: returns undefined
 * - NaNToUndefined mode: returns undefined
 *
 * @param x Input to be converted
 * @param {TypeParser_Options} (options)
 * @return {Number|undefined}
 */
module.exports = function (x, options = {}) {
    const mode = ((options.mode || 'strict') + '').toLowerCase();

    // Warning message for unknown modes
    if (!['strict', 'nanto0', 'nantoundefined'].includes(mode)) {
        throw {
            type: 'warning',
            message: `WARNING: Unknown mode "${mode}" used for toTimestamp`,
            method: 'toTimestamp',
        };
    }

    if (isValidDate(x)) {
        return x.getTime();
    }

    if (isEmpty(x)) {
        if (mode === 'strict') {
            return NaN;
        }
        else if (mode === 'nanto0') {
            return 0;
        }
        return undefined;
    }
    else {

        let v = toNumber(x, { allowUndefined: true });
        if (!isNumber(v)) { v = x; }

        try {
            v = JSON.parse(v);
        }
        catch (e) {}

        const d = new Date(v);
        if (isValidDate(d)) {
            return d.getTime();
        }
        else {
            if (mode === 'strict') {
                return NaN;
            }
            else if (mode === 'nanto0') {
                return 0;
            }
            return undefined;
        }
    }
};