const assert = require('assert');

const T = require('../../index');

/**
 * @name T
 * @property {TypeParser_Type} TEST_CASES_Type1
 * @property {TypeParser_Type} TEST_CASES_Type2
 * @property {TypeParser_Type} TEST_CASES_Type3
 * @property {TypeParser_Type} TEST_CASES_Type4
 * @property {TypeParser_Type} TEST_CASES_Type5
 * @property {TypeParser_Type} TEST_CASES_Type6
 */

module.exports = function () {
    describe('Testing new types', () => {

        before('Registering new types', () => {

            // Regular type
            T.register({
                title: 'TEST_CASES_Type1',
                from: function (v) {
                    return 'TYPE1_' + v;
                },
                isTypeOf: function (v) { {
                    return v === 'TYPE1';
                }},
            });

            // Regular type, no IS
            T.register({
                title: 'TEST_CASES_Type2',
                from: function (v) {
                    return 'TYPE2_' + v;
                },
            });

            // Regular type, no TO
            T.register({
                title: 'TEST_CASES_Type3',
                isTypeOf: function (v) { {
                    return v === 'TYPE3';
                }},
            });

            // Regular type, with options
            T.register({
                title: 'TEST_CASES_Type4',
                from: function (v, options) {
                    return 'TYPE4_' + options.s + '_' + v;
                },
                isTypeOf: function (v, options) { {
                    return v === 'TYPE4_' + options.s;
                }},
            });

            // Regular type, asynchronous
            T.register({
                title: 'TEST_CASES_Type5',
                requireCallback: true,
                from: function (v, options, cb) {
                    setTimeout(() => {
                        cb('TYPE5_' + v);
                    }, 200);
                },
                isTypeOf: function (v, options, cb) { {
                    setTimeout(() => {
                        cb(v === 'TYPE5');
                    }, 200);
                }},
            });

            // Regular type, asynchronous, with options
            T.register({
                title: 'TEST_CASES_Type6',
                requireCallback: true,
                from: function (v, options, cb) {
                    setTimeout(() => {
                        cb('TYPE6_' + options.s + '_' + v);
                    }, 200);
                },
                isTypeOf: function (v, options, cb) { {
                    setTimeout(() => {
                        cb(v === 'TYPE6_' + options.s);
                    }, 200);
                }},
            });
        });


        it('Testing Type 1 - TO', () => {
            const c1 = T.TEST_CASES_Type1.from('A') === 'TYPE1_A';
            const c2 = T.TEST_CASES_Type1.from('B') === 'TYPE1_B';
            const c3 = T.TEST_CASES_Type1.from('B') !== 'TYPE1_A';
            assert.equal(c1 && c2 && c3, true);
        });

        it('Testing Type 1 - IS', () => {
            const c1 =  T.TEST_CASES_Type1.isTypeOf('TYPE1');
            const c2 = !T.TEST_CASES_Type1.isTypeOf('TYPEZ');
            assert.equal(c1 && c2, true);
        });

        it('Testing Type 2 - TO', () => {
            const c1 = T.TEST_CASES_Type2.from('A') === 'TYPE2_A';
            const c2 = T.TEST_CASES_Type2.from('B') === 'TYPE2_B';
            const c3 = T.TEST_CASES_Type2.from('B') !== 'TYPE2_A';
            assert.equal(c1 && c2 && c3, true);
        });

        it('Testing Type 3 - IS', () => {
            const c1 =  T.TEST_CASES_Type3.isTypeOf('TYPE3');
            const c2 = !T.TEST_CASES_Type3.isTypeOf('TYPEZ');
            assert.equal(c1 && c2, true);
        });

        it('Testing Type 4 - TO', () => {
            const c1 = T.TEST_CASES_Type4.from('A', { s: 'B' }) === 'TYPE4_B_A';
            const c2 = T.TEST_CASES_Type4.from('B', { s: 'B' }) === 'TYPE4_B_B';
            const c3 = T.TEST_CASES_Type4.from('B', { s: 'B' }) !== 'TYPE4_B_A';
            assert.equal(c1 && c2 && c3, true);
        });

        it('Testing Type 4 - IS', () => {
            const c1 =  T.TEST_CASES_Type4.isTypeOf('TYPE4_B', { s: 'B' });
            const c2 = !T.TEST_CASES_Type4.isTypeOf('TYPE4_Z', { s: 'B' });
            assert.equal(c1 && c2, true);
        });

        it('Testing Type 5 - TO', (done) => {
            let c1;
            let c2;
            let c3;
            T.TEST_CASES_Type5.from('A', (result) => {
                c1 = result === 'TYPE5_A';
                T.TEST_CASES_Type5.from('B', (result) => {
                    c2 = result === 'TYPE5_B';
                    T.TEST_CASES_Type5.from('B', (result) => {
                        c3 = result !== 'TYPE5_A';
                        assert.equal(c1 && c2 && c3, true);
                        done()
                    });
                });
            });
        });

        it('Testing Type 5 - IS', (done) => {
            let c1;
            let c2;
            T.TEST_CASES_Type5.isTypeOf('TYPE5', (result) => {
                c1 = result;
                T.TEST_CASES_Type5.isTypeOf('TYPEZ', (result) => {
                    c2 = !result;
                    assert.equal(c1 && c2, true);
                    done()
                });
            });
        });

        it('Testing Type 6 - TO', (done) => {
            let c1;
            let c2;
            let c3;
            T.TEST_CASES_Type6.from('A', { s: 'B' }, (result) => {
                c1 = result === 'TYPE6_B_A';
                T.TEST_CASES_Type6.from('B', { s: 'B' }, (result) => {
                    c2 = result === 'TYPE6_B_B';
                    T.TEST_CASES_Type6.from('B', { s: 'B' }, (result) => {
                        c3 = result !== 'TYPE6_B_A';
                        assert.equal(c1 && c2 && c3, true);
                        done()
                    });
                });
            });
        });

        it('Testing Type 6 - IS', (done) => {
            let c1;
            let c2;
            T.TEST_CASES_Type6.isTypeOf('TYPE6_B', {s: 'B'}, (result) => {
                c1 = result;
                T.TEST_CASES_Type6.isTypeOf('TYPEZ_B', {s: 'B'}, (result) => {
                    c2 = !result;
                    assert.equal(c1 && c2, true);
                    done()
                });
            });
        });


        describe('Testing illegal calls', () => {

            it('Inverted Option<>Callback', (done) => {
                try {
                    T.TEST_CASES_Type6.isTypeOf('TYPE6_B', (result) => {
                        assert.fail('Failed');
                    }, {s: 'B'});
                }
                catch(e) {
                    assert.ok('Passed');
                    done();
                }
            });

            it('Two callbacks', (done) => {
                try {
                    T.TEST_CASES_Type6.isTypeOf('TYPE6_B', (result) => {
                        assert.fail('Failed');
                    }, (result) => {
                        assert.fail('Failed');
                    });
                }
                catch(e) {
                    assert.ok('Passed');
                    done();
                }
            });

            it('Two options', (done) => {
                try {
                    T.TEST_CASES_Type6.isTypeOf('TYPE6_B', {}, {});
                    assert.fail('Failed');
                }
                catch(e) {
                    assert.ok('Passed');
                    done();
                }
            });

            it('No callback', (done) => {
                try {
                    T.TEST_CASES_Type6.isTypeOf('TYPE6_B');
                    assert.fail('Failed');
                }
                catch(e) {
                    assert.ok('Passed');
                    done();
                }
            });

        });

    });
};