const assert = require('assert');

const isValidDate = require('../../src/validation/isValidDate');

const toTestTitle = require('../utils/toTestTitle');

const date = new Date();
const invalidDate = new Date("abc");
const emptyFunction = function () {};

module.exports = function (tester = isValidDate) {
    describe('isValidDate', () => {
        [
            // Numbers: integers and floats
            { test: 0,    expect: false },
            { test: -0,   expect: false },
            { test: 1,    expect: false },
            { test: -1,   expect: false },
            { test: 0.0,  expect: false },
            { test: -0.0, expect: false },
            { test: 0.1,  expect: false },
            { test: 1.1,  expect: false },
            { test: 1.9,  expect: false },
            { test: -1.1, expect: false },
            { test: -1.9, expect: false },

            // Null, NaN and undefined
            { test: null,      expect: false },
            { test: NaN,       expect: false },
            { test: undefined, expect: false },

            // Strings
            { test: 'true',                     expect: false },
            { test: 'false',                    expect: false },
            { test: 'abc',                      expect: false },
            { test: '',                         expect: false },
            { test: '0',                        expect: false },
            { test: '1',                        expect: false },
            { test: '-1',                       expect: false },
            { test: "",                         expect: false },
            { test: "abc",                      expect: false },
            { test: "[1,2,3]",                  expect: false },
            { test: '{"a":1, "b":3}',           expect: false },
            { test: "2017-02-24T19:08:42.835Z", expect: false },

            // Arrays
            { test: [],      expect: false },
            { test: [1,2,3], expect: false },
            { test: ["1"],   expect: false },

            // Objects
            { test: {a:1}, expect: false },
            { test: {},    expect: false },

            // Booleans
            { test: false, expect: false },
            { test: true,  expect: false },

            // Other
            { test: emptyFunction, expect: false },
            { test: date,          expect: true  },
            { test: invalidDate,   expect: false },
        ].forEach(({ test, expect }) => {
            it(`${toTestTitle(test)} => ${toTestTitle(expect)}`, () => {
                const result = tester(test);
                assert.deepEqual(result, expect);
            });
        });
    });
};