const assert = require('assert');

const toFunction = require('../../src/conversion/toFunction');

const toTestTitle = require('../utils/toTestTitle');
const encodeFunction = require('../utils/encodeFunction');

const date = new Date();
const invalidDate = new Date("abc");
const emptyFunction = function () {};

module.exports = function (tester = toFunction) {
    describe('toFunction - { mode: "strict" } ', () => {
        [
            // Numbers: integers and floats
            { test: 0,    expect: function () {} },
            { test: -0,   expect: function () {} },
            { test: 1,    expect: function () {} },
            { test: -1,   expect: function () {} },
            { test: 0.0,  expect: function () {} },
            { test: -0.0, expect: function () {} },
            { test: 0.1,  expect: function () {} },
            { test: 1.1,  expect: function () {} },
            { test: 1.9,  expect: function () {} },
            { test: -1.1, expect: function () {} },
            { test: -1.9, expect: function () {} },

            // Null, NaN and undefined
            { test: null,      expect: function () {} },
            { test: NaN,       expect: function () {} },
            { test: undefined, expect: function () {} },

            // Strings
            { test: 'true',                     expect: function () {} },
            { test: 'false',                    expect: function () {} },
            { test: 'abc',                      expect: function () {} },
            { test: '',                         expect: function () {} },
            { test: '0',                        expect: function () {} },
            { test: '1',                        expect: function () {} },
            { test: '-1',                       expect: function () {} },
            { test: "",                         expect: function () {} },
            { test: "abc",                      expect: function () {} },
            { test: "[1,2,3]",                  expect: function () {} },
            { test: '{"a":1, "b":3}',           expect: function () {} },
            { test: "2017-02-24T19:08:42.835Z", expect: function () {} },

            // Arrays
            { test: [],      expect: function () {} },
            { test: [1,2,3], expect: function () {} },
            { test: ["1"],   expect: function () {} },

            // Objects
            { test: {a:1}, expect: function () {} },
            { test: {},    expect: function () {} },

            // Booleans
            { test: false, expect: function () {} },
            { test: true,  expect: function () {} },

            // Other
            { test: emptyFunction, expect: emptyFunction  },
            { test: date,          expect: function () {} },
            { test: invalidDate,   expect: function () {} },
        ].forEach(({ test, expect }) => {
            it(`${toTestTitle(test)} => ${toTestTitle(expect)}`, () => {
                const result = tester(test);
                assert.deepEqual(encodeFunction(result), encodeFunction(expect));
            });
        });
    });

    describe('toFunction - { mode: "allowUndefined" } ', () => {
        [
            // Numbers: integers and floats
            { test: 0,    expect: undefined },
            { test: -0,   expect: undefined },
            { test: 1,    expect: undefined },
            { test: -1,   expect: undefined },
            { test: 0.0,  expect: undefined },
            { test: -0.0, expect: undefined },
            { test: 0.1,  expect: undefined },
            { test: 1.1,  expect: undefined },
            { test: 1.9,  expect: undefined },
            { test: -1.1, expect: undefined },
            { test: -1.9, expect: undefined },

            // Null, NaN and undefined
            { test: null,      expect: undefined },
            { test: NaN,       expect: undefined },
            { test: undefined, expect: undefined },

            // Strings
            { test: 'true',                     expect: undefined },
            { test: 'false',                    expect: undefined },
            { test: 'abc',                      expect: undefined },
            { test: '',                         expect: undefined },
            { test: '0',                        expect: undefined },
            { test: '1',                        expect: undefined },
            { test: '-1',                       expect: undefined },
            { test: "",                         expect: undefined },
            { test: "abc",                      expect: undefined },
            { test: "[1,2,3]",                  expect: undefined },
            { test: '{"a":1, "b":3}',           expect: undefined },
            { test: "2017-02-24T19:08:42.835Z", expect: undefined },

            // Arrays
            { test: [],      expect: undefined },
            { test: [1,2,3], expect: undefined },
            { test: ["1"],   expect: undefined },

            // Objects
            { test: {a:1}, expect: undefined },
            { test: {},    expect: undefined },

            // Booleans
            { test: false, expect: undefined },
            { test: true,  expect: undefined },

            // Other
            { test: emptyFunction, expect: emptyFunction },
            { test: date,          expect: undefined     },
            { test: invalidDate,   expect: undefined     },
        ].forEach(({ test, expect }) => {
            it(`${toTestTitle(test)} => ${toTestTitle(expect)}`, () => {
                const result = tester(test, { mode: 'allowUndefined' });
                assert.deepEqual(encodeFunction(result), encodeFunction(expect));
            });
        });
    });

};


