const assert = require('assert');

const toObject = require('../../src/conversion/toObject');

const toTestTitle = require('../utils/toTestTitle');

const date = new Date();
const invalidDate = new Date("abc");
const emptyFunction = function () {};

module.exports = function (tester = toObject) {

    describe('toObject - { mode: "strict" }', () => {
        [
            // Numbers: integers and floats
            { test: 0,    expect: {} },
            { test: -0,   expect: {} },
            { test: 1,    expect: {} },
            { test: -1,   expect: {} },
            { test: 0.0,  expect: {} },
            { test: -0.0, expect: {} },
            { test: 0.1,  expect: {} },
            { test: 1.1,  expect: {} },
            { test: 1.9,  expect: {} },
            { test: -1.1, expect: {} },
            { test: -1.9, expect: {} },

            // Null, NaN and undefined
            { test: null,      expect: {} },
            { test: NaN,       expect: {} },
            { test: undefined, expect: {} },

            // Strings
            { test: 'true',                     expect: {}             },
            { test: 'false',                    expect: {}             },
            { test: 'abc',                      expect: {}             },
            { test: '',                         expect: {}             },
            { test: '0',                        expect: {}             },
            { test: '1',                        expect: {}             },
            { test: '-1',                       expect: {}             },
            { test: "",                         expect: {}             },
            { test: "abc",                      expect: {}             },
            { test: "[1,2,3]",                  expect: {}             },
            { test: '{"a":1, "b":3}',           expect: {"a":1, "b":3} },
            { test: "2017-02-24T19:08:42.835Z", expect: {}             },

            // Arrays
            { test: [],      expect: {} },
            { test: [1,2,3], expect: {} },
            { test: ["1"],   expect: {} },

            // Objects
            { test: {a:1}, expect: {a:1} },
            { test: {},    expect: {}    },

            // Booleans
            { test: false, expect: {} },
            { test: true,  expect: {} },

            // Other
            { test: emptyFunction, expect: {}          },
            { test: date,          expect: date        },
            { test: invalidDate,   expect: invalidDate },
        ].forEach(({ test, expect }) => {
            it(`${toTestTitle(test)} => ${toTestTitle(expect)}`, () => {
                const result = tester(test);
                assert.deepEqual(result, expect);
            });
        });
    });

    describe('toObject - { mode: "allowUndefined" } ', () => {
        [
            // Numbers: integers and floats
            { test: 0,    expect: undefined },
            { test: -0,   expect: undefined },
            { test: 1,    expect: undefined },
            { test: -1,   expect: undefined },
            { test: 0.0,  expect: undefined },
            { test: -0.0, expect: undefined },
            { test: 0.1,  expect: undefined },
            { test: 1.1,  expect: undefined },
            { test: 1.9,  expect: undefined },
            { test: -1.1, expect: undefined },
            { test: -1.9, expect: undefined },

            // Null, NaN and undefined
            { test: null,      expect: undefined },
            { test: NaN,       expect: undefined },
            { test: undefined, expect: undefined },

            // Strings
            { test: 'true',                     expect: undefined      },
            { test: 'false',                    expect: undefined      },
            { test: 'abc',                      expect: undefined      },
            { test: '',                         expect: undefined      },
            { test: '0',                        expect: undefined      },
            { test: '1',                        expect: undefined      },
            { test: '-1',                       expect: undefined      },
            { test: "",                         expect: undefined      },
            { test: "abc",                      expect: undefined      },
            { test: "[1,2,3]",                  expect: undefined      },
            { test: '{"a":1, "b":3}',           expect: {"a":1, "b":3} },
            { test: "2017-02-24T19:08:42.835Z", expect: undefined      },

            // Arrays
            { test: [],      expect: undefined },
            { test: [1,2,3], expect: undefined },
            { test: ["1"],   expect: undefined },

            // Objects
            { test: {a:1}, expect: {a:1} },
            { test: {},    expect: {}    },

            // Booleans
            { test: false, expect: undefined },
            { test: true,  expect: undefined },

            // Other
            { test: emptyFunction, expect: undefined   },
            { test: date,          expect: date        },
            { test: invalidDate,   expect: invalidDate },
        ].forEach(({ test, expect }) => {
            it(`${toTestTitle(test)} => ${toTestTitle(expect)}`, () => {
                const result = tester(test, { mode: 'allowUndefined' });
                assert.deepEqual(result, expect);
            });
        });
    });
};


